#include "NTAItem.h"
#include <iostream>
#include <stdexcept>

using namespace refinitiv::ema::access;
using namespace refinitiv::ema::rdm;
using namespace std;

static const Int16 GUID_FID = 4271;
static const Int16 MRN_SRC = 12215;
static const Int16 MRN_TYPE = 8593;
static const Int16 FRAG_NUM = 32479;
static const Int16 FRAGMENT = 32641;
static const Int16 TOT_SIZE = 32480;

NTAItem::NTAItem()
{
}


NTAItem::~NTAItem()
{
}

// Initialise member variables with Update field list content by extracting data
// from the corresponding Fields
void NTAItem::init(const refinitiv::ema::access::FieldList& fl)
{
	Int16 fidNum = 0;
	UInt64 tmpUint = 0;
	EmaString tmpString;

	// Need to reset the iterator as we have already worked through the list for the decode() method
	fl.reset();	

	while (fl.forth())	
	{
		const FieldEntry& fe = fl.getEntry();
		//Get the Field ID for this field
		fidNum = fe.getFieldId();	

		// We are only interested in a few key fields 
		// so we only need to handle the corresponding field types
		switch (fe.getLoadType())
		{
			case DataType::UIntEnum:
				tmpUint = fe.getUInt();
				cout << tmpUint << endl;
				if (fidNum == FRAG_NUM)		// Fragment Number
					_fragNum = tmpUint;
				else if (fidNum == TOT_SIZE)	// Expected total size of all fragments
					_expectedSize = tmpUint;
				break;
			case DataType::RmtesEnum:
				tmpString = fe.getRmtes().toString();
				if (fidNum == GUID_FID)			// The Unique (source specific) ID for the NTA Item
					_guid = tmpString;
				else if (fidNum == MRN_SRC)		// The component which published the data
					_mrnSource = tmpString;
				else if (fidNum == MRN_TYPE)	// The type of NTA Item e.g Story, Headline etc
					_mrnType = tmpString;
				break;
			case DataType::BufferEnum:
				if (fidNum == FRAGMENT)			// The zipped Fragment buffer
					_ntaBuffer.append(fe.getBuffer().c_buf(), fe.getBuffer().length());
				break;
			default:
				break;
		}
	}

	if (_fragNum==1)
		std::cout << "Init:: Expected total buffer size:" << _expectedSize << " current size:" << _ntaBuffer.size() << std::endl;
	else
		std::cout << "Addtional fragment size:" << _ntaBuffer.size() << std::endl;
}

// Multi Fragment items - add a new fragment to the existing fragment(s)
void NTAItem::addFragment(const NTAItem& fragment)
{
	if (fragment._guid != _guid)
		throw std::runtime_error("Cannot add fragment to item: mismatching GUID");
	if (fragment._mrnSource != _mrnSource) 
		throw std::runtime_error("Cannot add fragment to item: mismatching data source");
	if (fragment._fragNum != _fragNum + 1) 
		throw std::runtime_error("Cannot add fragment to item: fragment number is not in sequence");

	_fragNum = fragment._fragNum;
	_ntaBuffer.append(fragment._ntaBuffer.begin(), fragment._ntaBuffer.end());

	cout << "Add Fragment:: Expected total buffer:" << _expectedSize << " current " << _ntaBuffer.size() << std::endl;
}

